package com.mortals.xhx.module.api;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.web.BaseJsonBodyController;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.certificate.model.CertificateCatalogEntity;
import com.mortals.xhx.module.certificate.model.CertificateCatalogQuery;
import com.mortals.xhx.module.certificate.model.CertificateCatalogTemplateEntity;
import com.mortals.xhx.module.certificate.model.CertificateCatalogTemplateQuery;
import com.mortals.xhx.module.certificate.pdu.ApplyLogPdu;
import com.mortals.xhx.module.certificate.pdu.PrintListPdu;
import com.mortals.xhx.module.certificate.pdu.PrintLogPdu;
import com.mortals.xhx.module.certificate.service.CertificateCatalogService;
import com.mortals.xhx.module.certificate.service.CertificateCatalogTemplateService;
import com.mortals.xhx.module.print.model.PrintCatalogEntity;
import com.mortals.xhx.module.print.model.PrintCatalogQuery;
import com.mortals.xhx.module.print.model.PrintServiceEntity;
import com.mortals.xhx.module.print.service.PrintCatalogService;
import com.mortals.xhx.module.print.service.PrintServiceService;
import com.mortals.xhx.module.record.model.PrintWaitQueueEntity;
import com.mortals.xhx.module.record.model.PrintWaitQueueQuery;
import com.mortals.xhx.module.record.service.ApplyLogService;
import com.mortals.xhx.module.record.service.PrintWaitQueueService;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("api")
public class CertificateApi extends BaseJsonBodyController {

    @Autowired
    private CertificateCatalogService certificateCatalogService;
    @Autowired
    private ApplyLogService applyLogService;
    @Autowired
    private PrintServiceService printServiceService;
    @Autowired
    private PrintCatalogService printCatalogService;
    @Autowired
    private PrintWaitQueueService printWaitQueueService;
    @Autowired
    private CertificateCatalogTemplateService certificateCatalogTemplateService;

    @PostMapping({"catalog/list"})
    @UnAuth
    public String getCatalogList(){

        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap();
        String busiDesc = "API接口查询证照目录列表";
        byte code = 1;
        try {
            List<CertificateCatalogEntity> list = certificateCatalogService.find(new CertificateCatalogQuery());

            if(CollectionUtils.isNotEmpty(list)){
                List<CatalogVO> catalogList = new ArrayList<>();
                for(CertificateCatalogEntity entity:list){

                    List<CertificateCatalogTemplateEntity> templateEntityList = certificateCatalogTemplateService.find(new CertificateCatalogTemplateQuery().catalogId(entity.getId()));
                    if(CollectionUtils.isNotEmpty(templateEntityList)){

                        for (CertificateCatalogTemplateEntity item:templateEntityList){
                            CatalogVO catalogVO = new CatalogVO(entity.getId(),entity.getCatalogName(),entity.getCatalogCode());
                            catalogVO.setTemplateId(item.getId());
                            catalogVO.setTemplateName(item.getTemplateName());
                            JSONObject formContent = JSONObject.parseObject(item.getFormContent());
                            JSONArray formList = formContent.getJSONArray("list");
                            JSONObject json = new JSONObject();
                            for (int j = 0; j < formList.size(); j++) {
                                JSONObject jsonObject = formList.getJSONObject(j);
                                json.put(jsonObject.getString("id"),"");
                            }
                            catalogVO.setFormContent(json.toJSONString());

                            catalogList.add(catalogVO);
                        }
                    }

                }
                model.put("message_info", "获取证照目录成功");
                ret.put("data", catalogList);
            }else {
                code = 0;
                model.put("message_info", "没有数据");
            }

            this.recordSysLog(this.request, busiDesc + " 【成功】");
        }catch (Exception e){
            code = -1;
            model.put("message_info", "系统发生异常,请稍后再试");
            this.doException(this.request, busiDesc, model, e);
        }
        ret.put("code",  Integer.valueOf(code));
        ret.put("msg", model.remove("message_info"));
        return ret.toJSONString();
    }

    @PostMapping({"certificate/submit"})
    @UnAuth
    public String postA(@RequestBody ApplyLogPdu applyLogPdu){
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap();
        byte code = 1;
        String busiDesc = "API接口生成证照";
        try {
            applyLogService.apiSaveApplyLog(applyLogPdu);
            model.put("message_info", "生成证照成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        }catch (Exception e){
            code = -1;
            model.put("message_info", "生成证照发生异常");
            if(e instanceof AppException){
                model.put("message_info", e.getMessage());
            }
            this.doException(this.request, busiDesc, model, e);
        }
        ret.put("code",  Integer.valueOf(code));
        ret.put("msg", model.remove("message_info"));
        return ret.toJSONString();
    }

    @PostMapping({"print/service/list"})
    @UnAuth
    public String printServiceList(){
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap();
        String busiDesc = "API接口查询打印服务配置列表";
        byte code = 1;
        try {
            List<PrintServiceEntity> list = printServiceService.find(new PrintServiceEntity());
            if(CollectionUtils.isNotEmpty(list)){
                List<PrintServiceVO> serviceList = new ArrayList<>();
                for(PrintServiceEntity entity:list){
                    PrintServiceVO printServiceVO = new PrintServiceVO(entity.getId(),entity.getServiceName(),entity.getServiceUrl());
                    serviceList.add(printServiceVO);
                }
                model.put("message_info", "获取打印服务配置列表成功");
                ret.put("data", serviceList);
            }else {
                code = 0;
                model.put("message_info", "没有数据");
            }
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        }catch (Exception e){
            code = -1;
            model.put("message_info", "系统发生异常,请稍后再试");
            this.doException(this.request, busiDesc, model, e);
        }
        ret.put("code",  Integer.valueOf(code));
        ret.put("msg", model.remove("message_info"));
        return ret.toJSONString();
    }

    @PostMapping({"print/catalog/service"})
    @UnAuth
    public String printCatalogService(@RequestBody PrintServicePdu printServicePdu){
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap();
        String busiDesc = "API接口查询目录打印配置";
        byte code = 1;
        try {
            if(printServicePdu.getServiceId()==null){
                throw new AppException("打印服务配置ID不能为空");
            }
            if(printServicePdu.getCatalogId()==null){
                throw new AppException("证照目录ID不能为空");
            }
            PrintCatalogQuery query = new PrintCatalogQuery();
            query.setServiceId(printServicePdu.getServiceId());
            query.setCatalogId(printServicePdu.getCatalogId());
            PrintCatalogVO printCatalogVO = null;
            List<PrintCatalogEntity> list = printCatalogService.find(query);
            if(CollectionUtils.isNotEmpty(list)&&list.size()>0){
                printCatalogVO = new PrintCatalogVO(list.get(0).getPrinterName(),list.get(0).getPrinterIp(),list.get(0).getPaperSource());
                model.put("message_info", "获取目录打印配置成功");
                ret.put("data", printCatalogVO);
            }else {
                code = 0;
                model.put("message_info", "没有数据");
            }

            this.recordSysLog(this.request, busiDesc + " 【成功】");
        }catch (Exception e){
            code = -1;
            model.put("message_info", "系统发生异常,请稍后再试");
            this.doException(this.request, busiDesc, model, e);
        }
        ret.put("code",  Integer.valueOf(code));
        ret.put("msg", model.remove("message_info"));
        return ret.toJSONString();
    }


    /**
     * 获取打印文件
     */
    @PostMapping({"print/certificate/list"})
    @UnAuth
    public Rest<Object> listByIDCard(@RequestBody PrintListPdu pdu) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "API接口查询待打印证件";
        int code=1;
        try {
            if(pdu.getServiceId()==null){
                throw new AppException("打印服务配置ID不能为空");
            }
            PrintWaitQueueQuery query = new PrintWaitQueueQuery();
            query.setPrintStatus(YesNoEnum.NO.getValue());
            query.setPickerIDCardNo(pdu.getIdCard());
            List<PrintWaitQueueEntity> list = printWaitQueueService.find(query);
            if(CollectionUtils.isNotEmpty(list)){
                List<PrintWaitQueueVO> waitQueueVOList = new ArrayList<>();
                for(PrintWaitQueueEntity printWaitQueueEntity:list){
                    PrintCatalogEntity printCatalogEntity = printCatalogService.getPrintCatalogByService(pdu.getServiceId(),printWaitQueueEntity.getCatalogId(),printWaitQueueEntity.getTemplateId());
                    PrintWaitQueueVO printWaitQueueVO = new PrintWaitQueueVO();
                    printWaitQueueVO.setCatalogId(printWaitQueueEntity.getCatalogId());
                    printWaitQueueVO.setCatalogCode(printWaitQueueEntity.getCatalogCode());
                    printWaitQueueVO.setCatalogName(printWaitQueueEntity.getCatalogName());
                    printWaitQueueVO.setTemplateId(printWaitQueueEntity.getTemplateId());
                    printWaitQueueVO.setTemplateName(printWaitQueueEntity.getTemplateName());
                    printWaitQueueVO.setWaitQueueId(printWaitQueueEntity.getId());
                    printWaitQueueVO.setCertificateCode(printWaitQueueEntity.getCertificateCode());
                    printWaitQueueVO.setCertificateName(printWaitQueueEntity.getCertificateName());
                    printWaitQueueVO.setCertificateUrl(printWaitQueueEntity.getCertificateUrl());
                    printWaitQueueVO.setEnterpriseName(printWaitQueueEntity.getEnterpriseName());
                    printWaitQueueVO.setHolderIDCardNo(printWaitQueueEntity.getHolderIDCardNo());
                    printWaitQueueVO.setHolderName(printWaitQueueEntity.getHolderName());
                    printWaitQueueVO.setPickerIDCardNo(printWaitQueueEntity.getPickerIDCardNo());
                    printWaitQueueVO.setPickerName(printWaitQueueEntity.getPickerName());
                    printWaitQueueVO.setPreviewUrl(printWaitQueueEntity.getPreviewUrl());
                    printWaitQueueVO.setServiceUrl(printCatalogEntity.getServiceUrl());
                    printWaitQueueVO.setPrinterIp(printCatalogEntity.getPrinterIp());
                    printWaitQueueVO.setPrinterName(printCatalogEntity.getPrinterName());
                    printWaitQueueVO.setPaperSource(printCatalogEntity.getPaperSource());
                    waitQueueVOList.add(printWaitQueueVO);
                }
                model.put("data", waitQueueVOList);
                model.put("message_info", busiDesc + "成功");
            }else {
                code = 0;
                model.put("message_info", "没有数据");
            }

            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

    /**
     * 打印文件
     */
    @RequestMapping(value = {"print/certificate/log"},method = {RequestMethod.POST, RequestMethod.GET})
    @UnAuth
    public Rest<Object> print(@RequestBody PrintLogPdu printLogPdu) {
        Rest<Object> ret = new Rest();
        Map<String, Object> model = new HashMap();
        String busiDesc = "API接口保存打印记录";
        int code=1;
        try {
            printWaitQueueService.doPrint(printLogPdu.getWaitQueueId(),printLogPdu.getPrintStatus(),printLogPdu.getStatusRemark());
            model.put("message_info", busiDesc + "成功");
            this.recordSysLog(this.request, busiDesc + " 【成功】");
        } catch (Exception var9) {
            code = -1;
            this.doException(this.request, busiDesc, model, var9);
        }
        ret.setCode(code);
        ret.setData(model);
        ret.setDict(model.get("dict"));
        ret.setMsg(model.get("message_info") == null ? "" : model.remove("message_info").toString());
        return ret;
    }

}
